# コード1.5
library(glmnet)

# データの読み込みと基準化
url<- "https://archive.ics.uci.edu/ml/machine-learning-databases/concrete/slump/slump_test.data"
data<- read.csv(url, header=TRUE)[, -1]

x<- data[, 1:7]
y<- data[, 9]

x<- scale(x)
y<- y-mean(y)

# 10分割交差検証法によるCV誤差のプロット
cv.lasso <- cv.glmnet(x, y, family = "gaussian", alpha = 1, nfolds = 10)

plot(cv.lasso, xlab="Log Lambda", ylab="CV Error", cex.axis=1.5, cex.lab=1.5)
grid()

# 1一つ抜き交差検証法の場合は次のようにすると良い
# cv.lasso<- cv.glmnet(x, y, family = "gaussian", alpha=1, nfolds=nrow(x), grouped=FALSE)

# 選択したlambdaを用いたときのラッソ推定値
# CV誤差を最小にするlambdaを用いた場合
lambda.min<- cv.lasso$lambda.min
glmnet(x, y, family = "gaussian", lambda = lambda.min , alpha = 1)$beta

# 1標準誤差基準によるlambda を用いた場合
lambda.1se<- cv.lasso$lambda.1se
glmnet(x, y, family = "gaussian", lambda = lambda.1se , alpha = 1)$beta




