##### コード3.7
# Python3.6を使用するための環境の準備
# 引数には, python3.6の置いてあるパスを指定する
Sys.setenv(TENSORFLOW_PYTHON = "/anaconda3/bin/python3.6")

library(tensorflow)
library(keras)

mnist <- dataset_mnist()
x_train <- mnist$train$x
x_test <- mnist$test$x

# 入力を区間[0,1]に値をとるようスケーリング(入力は28*28行列)
x_train <- array_reshape(x_train, c(dim(x_train), 1)) / 255
x_test <- array_reshape(x_test, c(dim(x_test), 1)) / 255

# 正解ラベルをone-of-K表記へ変換
y_train <- to_categorical(mnist$train$y, 10)
y_test <- to_categorical(mnist$test$y, 10)

##### モデルの定義
## スキップコネクション(コード3.4), ドロップアウト(コード3.5), バッチ正規化(コード3.6)を利用する場合, 
## 関数layer_add(), layer_dropout(), layer_batch_normalization()を利用する
x<- layer_input(c(28, 28, 1))
z1<- x %>% layer_conv_2d(filters = 64, kernel_size = c(3, 3), padding = "same")
act1<- z1 %>% layer_activation("relu")
z2<- act1 %>% layer_max_pooling_2d(pool_size=c(2, 2))
z3<- z2 %>% layer_conv_2d(filters = 32, kernel_size = c(3, 3), padding = "same")
act2<- z3 %>% layer_activation("relu")
z4<- act2 %>% layer_max_pooling_2d(pool_size=c(2, 2))
flatten<- z4 %>% layer_flatten()
z5<- flatten %>% layer_dense(units = 128, activation = "relu")
z6<- z5 %>% layer_dense(units = 64, activation = "relu")
y<- z6 %>% layer_dense(units = 10, activation = "softmax")

model<- keras_model(x, y)
model

## メソッドの指定とパラメータ推定
model %>% compile(
  loss = "categorical_crossentropy",
  optimizer = "rmsprop",
  metrics = c("accuracy")
)

history <- model %>% fit(
  x_train, y_train, 
  epochs = 30, batch_size = 128, 
  validation_data=list(x_test, y_test)
)

# 当てはめ結果のプロット(誤差関数の値と分類精度)
plot(history)

##### 分類精度の混同行列
# 対角成分は正解ラベル数を表している
predict<- model %>% predict(x_test)
predict_label<- apply(predict, 1, function(x){which(x==max(x))})-1

tab<- table(mnist$test$y, predict_label)
tab


##################
## 中間層の可視化(コード3.8)
## 123番目の訓練データに対する畳み込み層の可視化の例
k<- 123
X<- array_reshape(x_train[k,,,], c(1, dim(x_train[k,,,]), 1))

# 1つ目の畳み込み層での変換
## outputs　= model$get_layer("conv2d_1")$output とすると, 2つ目の畳み込み層での変換が得られる
## 今の場合, 1つ目の畳み込み層の出力Zはc(1, 28, 28, 64)の配列, 2つ目の畳み込み層の出力はc(1, 14, 14, 32)の配列となる
## modelの出力を参照されたい
layer<- keras_model(inputs=model$input, outputs=model$get_layer("conv2d")$output)
Z<- layer %>% predict(X)
img<- apply(Z[1,,,], 3, function(x){matrix((x-min(x))/max(x-min(x)), 28, 28)})

## imgの1列目を画像として可視化
plot(as.raster(matrix(img[, 10], 28, 28)))

