##### データの取得(コード3.1)
# Python3.6を使用するための環境の準備
# 引数には, python3.6の置いてあるパスを指定する
Sys.setenv(TENSORFLOW_PYTHON = "/anaconda3/bin/python3.6")

library(tensorflow)
library(keras)

mnist <- dataset_mnist()
x_train <- mnist$train$x
y_train <- mnist$train$y
x_test <- mnist$test$x
y_test <- mnist$test$y

# 入力をベクトルに変換し，区間[0,1]に値をとるようスケーリング
x_train <- array_reshape(x_train, c(nrow(x_train), 784))
x_test <- array_reshape(x_test, c(nrow(x_test), 784))
x_train <- x_train / 255
x_test <- x_test / 255

# 正解ラベルをone-of-K表記へ変換
y_train <- to_categorical(y_train, 10)
y_test <- to_categorical(y_test, 10)


##### モデルの定義(コード3.2)
## functional API によるモデルの定義
x<- layer_input(784)
z1<- x %>% layer_dense(units = 256, activation="relu")
z2<- z1 %>% layer_dense(units = 128, activation="relu")
y<- z2 %>% layer_dense(units = 10, activation="softmax")
model<- keras_model(x, y)
model

## 上記のモデルは, 次のように定義することもできる
# model <- keras_model_sequential()
# model %>%
#   layer_dense(units = 256, activation="relu", input_shape = c(784)) %>%
#   layer_dense(units = 128, activation="relu") %>%
#   layer_dense(units = 10, activation="softmax")


##### メソッドの指定とパラメータ推定(コード3.3)
model %>% compile(
  loss = "categorical_crossentropy",
  optimizer = "sgd",
  metrics = c("accuracy")
)

history <- model %>% fit(
  x_train, y_train, 
  epochs = 30, batch_size = 1, 
  validation_data=list(x_test, y_test)
)

# 当てはめ結果のプロット(誤差関数の値と分類精度)
plot(history)

##### 分類精度の混同行列
# 対角成分は正解ラベル数を表している
predict<- model %>% predict(x_test)
predict_label<- apply(predict, 1, function(x){which(x==max(x))})-1

tab<- table(mnist$test$y, predict_label)
tab




